#!/usr/bin/perl
use strict;

my %b1=();
my %repl=();
my %map=();

sub add {
 my ($x,$y)=@_;
 $b1{$x}=$y;
 $repl{$y}=$x;
}

sub alias {
 my ($x,$y)=@_;
 $repl{$x}=$y;
}

sub map_add {
 my ($f,$x,$y)=@_;
 $map{$f.'_'.$x}=$y;
 $repl{$y}=$x;
}

add('00','property');
add('01','define_sprite');
add('02','define_set_id');
add('03','associate_vehicle');
add('04','define_vehicle');
add('05','define_patch_set');
add('06','modify_next_sprite');
add('07','skip_init');
add('08','define_id');
add('09','skip');
add('0A','modify_ttd_sprite');
add('0B','error');
add('0C','nop');
add('0D','assign');
add('11','define_sounds');

sub parse_aliases {
 my ($a)=@_;
 my @t=split(/\n/,$a);
 foreach my $i (@t) {
  if ($i=~/([^=]*)=(.*)/) {
   alias($1,$2);
  }
 }
}

my $al_x=<<EOF;
load_amount=07
cargo_capacity=0D
running_cost_f=0F
cost_f=0A
ship_base_cost=13
ship_base_running_cost=2F
cost_base_multiplier=08
EOF

parse_aliases($al_x);

map_add('F','00','f_train');
map_add('F','01','f_road');
map_add('F','02','f_ship');
map_add('F','03','f_plane');
map_add('F','04','f_station');
map_add('F','05','f_canal');
map_add('F','07','f_house');
map_add('F','08','f_global');
map_add('F','09','f_industry_tile');
map_add('F','0A','f_industry');
map_add('F','0B','f_cargo');
map_add('F','0C','f_sound');

sub parse {
 my ($id,$txt)=@_;
 my $e='';
 $txt=~s/^\s+//;
 while ($id=~s/^(.)//) {
  my $t=$1;
  if ($id=~/^\+/) {
   $id="$t$id";
  }
  $txt=~s/^\s+//;
  last if ($txt eq '');
  if ($t eq 'W') {
   if ($txt=~s/([a-z0-9]+)\s+([a-z0-9]+)\s*//i) {
    my $n=hex($1)+256*hex($2);
    $e.="\\w$n ";
   } else {
    last;
   }
  } elsif ($t eq 'B') {
   if ($txt=~s/([a-z0-9]+)\s*//i) {
    my $n=hex($1);
    $e.="\\b$n ";
   } else {
    last;
   }
  } elsif ($t eq 'F') {
   if ($txt=~s/([a-z0-9]+)\s*//i) {
    my $ac=$1;
    my $k=$map{"${t}_${ac}"};
    if (!$k) { $k=$ac; }
    $e.="$k ";
   } else {
    last;
   }
  } else {
   die "Invalid FMT: $t";
  }
 }
 return $e.$txt;
}

my %parse_str=();

$parse_str{'00'}='FBB';
$parse_str{'01'}='F';
#$parse_str{'02'}='F';
$parse_str{'02'}='FBBBW+';
$parse_str{'03'}='F';
$parse_str{'04'}='F';
$parse_str{'0A'}='BBW+';
$parse_str{'11'}='W';

sub decode {
 my ($x)=@_;
 my $e='';
 $x=~s/^\s+//;
 my $ac='?';
 if ($x=~s/^([a-z0-9]+)//i) {
  $ac=$1;
  my $k=$b1{$ac};
  if (!$k) { $k=$ac; }
  $e.="$k ";
 }
 my $ps=$parse_str{$ac};
 if ($ps) {
  return $e.parse($ps,$x);
 }
 return $e.$x;
}

if ($ARGV[0] eq "-d") {

while(<STDIN>){
 s/[\r\n]+//g;
 if (s/^\s*(\d+)\s+\*\s+(\d+)\s+//) {
  my $x="$1\tDATA\t".decode($_);
  $_=$x;
 }
 if (s/^\s+(\d+)\s+\*\*\s+//) {
  my $x="$1\t**\t";
  $_=$x.$_;
 }
 if (s/^\s+(\d+)\s+//) {
  my $x="$1\t";
  $_=$x.$_;
 }
 print "$_\n";
}

} elsif ($ARGV[0] eq "-c") {

  foreach my $i (sort keys(%repl)) {
   my $s=$i;
   my $r=$repl{$i};
   print "$s = $r\n";
  }

} else {

 print <<EOF;
// Automatically generated by nfo_compile.pl. Do not modify!
// (Info version 7)
// Format: spritenum pcxfile xpos ypos compression ysize xsize xrel yrel
EOF

 my $serial=0;
 my $c='';
 my $use_count=0;
 $repl{'DATA'}='* 0';
 while(<STDIN>){
  if (/\@count/) {
   $use_count=1;
   next;
  }
  s/[\r\n]+//g;
  foreach my $i (keys(%repl)) {
   my $s=$i;
   my $r=$repl{$i};
   s/(^|[^a-z_0-9])$s/\1$r/;
  }
  s/\/\/.*//;
  last if (/EOF/);
  if (/^\s*\+/) {
   $serial++;
   s/^\s*\+/$serial/;
  }
  if (/^(\d+)/) {
   $serial=$1;
  }
  if (/./) {
   $c.="$_\n";
  }
 }
 if ($use_count) {
  $c="0	* 4	\\d$serial\n".$c;
 }
 print $c;
}